<?php

/**
 * @copyright	Copyright (C) 2010 Michael Richey. All rights reserved.
 * @license		GNU General Public License version 2 or later; see LICENSE.txt
 */
// no direct access
defined('_JEXEC') or die;

jimport('joomla.plugin.plugin');

class plgSystemExpiresHeaders extends JPlugin {

    private $app;
    private $site = false;
    private $expireInfo = null;
    private $itemid = null;
    private $activeitem = null;
    private $defaultitem = null;
    private $expired = false;
    private $itemrules = array();
    private $defaultrules = array();

    function __construct(&$subject, $config = array()) {
        parent::__construct($subject, $config);
        $this->app = JFactory::getApplication();
        if($this->site = $this->app->isSite()) {
            $this->activeitem = $this->app->getMenu()->getActive();
            $this->itemid = $this->activeitem?$this->activeitem->id:null;
            $this->defaultitem = $this->app->getMenu()->getDefault();
            $this->getExpireItems();
            $this->getDefaults();
        }
    }
    function onAfterInitialise() {
        if ($this->site && $this->itemid) {
            $this->calcExpire();
            $this->expire($this->expireInfo);
        }
    }

    function onAfterDispatch() {      
        if ($this->site && !$this->expireInfo && !$this->expired) {
            $this->activeitem = $this->app->getMenu()->getActive();
            $this->itemid = $this->activeitem->id;
            $this->calcExpire();
            $this->expire($this->expireInfo);
        }
    }

    function onAfterRender() {       
        if ($this->site) {
            if(!$this->expired) {
                $this->expire($this->expireInfo);
            }
        }
    }

    function calcExpire() {
        // only operate in the site
        if ($this->app->isAdmin()) return;
        
        // if the itemid is 0 and active menu is null, we're on the homepage - setting it to the default id from the menu
        $uri = JURI::getInstance();
        if (
        // these first two indicate that we might be on the homepage where there is no itemid
                $this->itemid == 0 &&
                is_null($this->app->getMenu()->getActive()) &&
                // we passed the first two tests, now two more to confirm homepage
                (
                    $uri->current() == $uri->base() ||
                    $uri->current() == $this->defaultitem->link
                )
        ) {
            $this->itemid = $this->defaultitem->id;
            $this->app->getMenu()->setActive($this->itemid);
        }

        // does this page have a rule, and does that rule indicate modification
        if ($this->itemid && in_array($this->itemid, array_keys($this->itemrules))) {            
            $this->expireInfo = $this->itemrules[$this->itemid];
        } else {
            if($this->defaultrules['expires']) {
                $this->expireInfo = $this->defaultrules;
            }
        }
    }

    function expire($vars) {
        if (!$vars) return null;
        if ($vars['expires']) {
            // allow caching
            // set the target date
            date_default_timezone_set('GMT');
            $cachetime = $vars['seconds']/60;
            $date = new DateTime();
            $expireheader = $date->setTimestamp($vars['time'])->format('D, j M Y H:i:s T');
            $this->app->allowCache(true);
            $this->setHeader('Expires', $expireheader,true);
//            die('<pre>'.print_r($this->app->get('response'),true).'</pre>');
        }
        // do we include the cache-control header
        if ($vars['cachecontrol']) {
            $valuearray = array();
            switch ($vars['publicprivate']) {
                case 0: $valuearray[] = 'public';
                    break;
                case 1: $valuearray[] = 'private';
                    break;
                case 2: $valuearray[] = 'no-cache';
                    break;
            }
            // should we set expire lengths?
            if ($vars['publicprivate'] < 2 && $vars['nostore'] != 1) {
                $valuearray[] = 'max-age=' . $vars['seconds'];
                $valuearray[] = 'pre-check=' . $vars['seconds'];
                $valuearray[] = 'post-check=' . $vars['seconds'];
            } else {
                $valuearray[] = 'max-age=0';
                $valuearray[] = 'pre-check=0';
                $valuearray[] = 'post-check=0';
            }
            if ($vars['nostore'])
                $valuearray[] = 'no-store';
            if ($vars['mustrevalidate'])
                $valuearray[] = 'must-revalidate';

            $this->setHeader('Cache-Control', implode(',', $valuearray));
        }
        // do we include pragma, and should we include it
        if ($vars['pragma'] && $vars['publicprivate'] == 2) {
            $this->setHeader('Pragma', 'no-cache');
        }
        $this->expired = true;
        return true;
    }

    private function setHeader($name,$value) {
        $this->app->setHeader($name,$value,true);
    }

    function getItemid() {
        return $this->activeitem?$this->activeitem->id:$this->defaultitem->id;
    }
    function getExpireItems(){
        date_default_timezone_set('GMT');
        $expireitems = json_decode($this->params->get('expireitems','{"ruleexpires":[],"ruleexpireslength":[],"ruleexpiresinterval":[],"rulecachecontrol":[],"rulepublicprivate":[],"rulenostore":[],"rulemustrevalidate":[],"rulepragma":[],"menulist":[]}'));
        foreach($expireitems->menulist as $key=>$menus) {
            $data = array();
            foreach(get_object_vars($expireitems) as $property=>$value) {
                switch($property) {
                    case 'menulist':
                        break;
                    default:
                        $data[str_replace('rule','',$property)]=$value[$key];
                        break;
                }
            }
            $itemvars = $this->itemvars($data);
            foreach($menus as $menu) {
                $this->itemrules[$menu]=$itemvars;
            }
        }
    }
    function getDefaults(){
        date_default_timezone_set('GMT');
        $data = array();
        $fields = array(
            'defaultexpires'=>0,
            'defaultexpireslength'=>2,
            'defaultexpiresinterval'=>'hour',
            'defaultcachecontrol'=>1,
            'defaultpublicprivate'=>0,
            'defaultnostore'=>'',
            'defaultmustrevalidate'=>'',
            'defaultpragma'=>''
        );
        foreach($fields as $varname=>$default) {
            $data[str_replace('default','',$varname)] = $this->params->get('',$default);
        }
        $this->defaultrules=$this->itemvars($data);
    }
    function itemvars($data) {
        $ret = array();
        date_default_timezone_set('GMT');
        $ret['expires']=(bool)$data['expires'];
        $ret['time'] = strtotime('+' . $data['expireslength'] . ' ' . $data['expiresinterval']);
        $ret['seconds'] = $ret['time'] - time();
        $ret['expireslength'] = (int) $data['expireslength'];
        $ret['expiresinterval'] = $data['expiresinterval'];
        $ret['cachecontrol'] = (bool) $data['cachecontrol'];
        $ret['nostore'] = ($data['nostore'] == '1') ? true : false;
        $ret['mustrevalidate'] = ($data['mustrevalidate'] == '1') ? true : false;
        $ret['publicprivate'] = (int) $data['publicprivate'];
        $ret['pragma'] = ($data['pragma'] == '1') ? true : false;
        return $ret;
    }
}
